#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_file.interfaces import IVideo, IThumbnails, IAudio, IMediaFile, IMediaWidget
from pyams_form.interfaces.form import IWidgetsPrefixViewletsManager, IFormHelp
from pyams_media.interfaces import IMediaConversionUtility, VIDEO_FRAME_SIZE, IMediaConverter, IMediaConversions
from pyams_media.zmi.interfaces import IMediaConversionsForm
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer
from transaction.interfaces import ITransactionManager
from z3c.form.interfaces import DISPLAY_MODE

# import packages
from pyams_form.form import AJAXEditForm
from pyams_form.help import FormHelp
from pyams_form.schema import CloseButton
from pyams_media.media import check_media_conversion
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarActionItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces import MANAGE_PERMISSION
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.form import AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, Interface

from pyams_media import _


class IMediaConversionsFormButtons(Interface):
    """Media conversions form buttons"""

    close = CloseButton(name='close', title=_("Cancel"))
    convert = button.Button(name='convert', title=_("Restart missing conversions"))


@adapter_config(context=(IMediaFile, IAdminLayer, IMediaConversionsForm), provides=IFormHelp)
class MediaConversionsFormHelpAdapter(FormHelp):
    """Medias conversions form help adapter"""

    message = _("""Several formats are required to handle different browsers and screen resolutions.
    
Conversion process may require some time based on media format and duration; this dashboard gives
you indications about required conversions.""")
    message_format = 'rest'


#
# Video conversions
#

@viewlet_config(name='video.conversions.action', context=IVideo, layer=IPyAMSLayer, view=IMediaWidget,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=15)
class VideoConversionsAction(ToolbarActionItem):
    """Video conversions action"""

    label = _("Video conversions...")
    label_css_class = 'fa fa-fw fa-compress'

    url = 'video-conversions.html'
    modal_target = True


@pagelet_config(name='video-conversions.html', context=IVideo, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
@implementer(IMediaConversionsForm)
class VideoConversionsEditForm(AdminDialogEditForm):
    """Video conversions display form"""

    prefix = 'video_conversion.'

    legend = _("Selected video conversions")
    icon_css_class = 'fa fa-fw fa-compress'

    fields = field.Fields(Interface)
    buttons = button.Buttons(IMediaConversionsFormButtons)
    edit_permission = MANAGE_PERMISSION
    ajax_handler = 'restart-video-conversions.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def check_mode(self):
        if not self.request.has_permission(self.edit_permission, self.getContent()):
            self.mode = DISPLAY_MODE

    def updateActions(self):
        super(VideoConversionsEditForm, self).updateActions()
        if 'convert' in self.actions:
            self.actions['convert'].addClass('btn-primary')

    def applyChanges(self, data):
        media = self.getContent()
        ITransactionManager(media).get().addAfterCommitHook(check_media_conversion, kws={'media': media})
        return {Interface: ()}


@view_config(name='restart-video-conversions.json', context=IVideo, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class VideoConversionsAJAXEditForm(AJAXEditForm, VideoConversionsEditForm):
    """Video conversions edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Media conversions restarted!"))
        }


@viewlet_config(name='video-conversions-prefix', context=IVideo, layer=IAdminLayer,
                view=VideoConversionsEditForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/video-conversions.pt')
class VideoConversionsViewletsPrefix(Viewlet):
    """Video conversions display form viewlets prefix"""

    @property
    def formats(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for format in utility.video_frame_size:
                yield format, VIDEO_FRAME_SIZE.get(format)

    @property
    def converters(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for converter_name in utility.video_formats:
                converter = query_utility(IMediaConverter, name=converter_name)
                if converter is not None:
                    yield converter_name, converter

    @property
    def original_frame_size(self):
        return IThumbnails(self.context).get_image_size()

    def has_conversion(self, converter, frame_size):
        conversions = IMediaConversions(self.context)
        name = 'video{width}.{extension}'.format(width='-{0}'.format(frame_size) if frame_size else '',
                                                 extension=converter.format)
        return name in conversions

    def get_conversion(self, converter, frame_size):
        conversions = IMediaConversions(self.context)
        name = 'video{width}.{extension}'.format(width='-{0}'.format(frame_size) if frame_size else '',
                                                 extension=converter.format)
        return conversions.get(name)


#
# Audio conversions
#

@viewlet_config(name='audio.conversions.action', context=IAudio, layer=IPyAMSLayer, view=IMediaWidget,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=15)
class AudioConversionsAction(ToolbarActionItem):
    """Audio conversions action"""

    label = _("Audio conversions...")
    label_css_class = 'fa fa-fw fa-compress'

    url = 'audio-conversions.html'
    modal_target = True


@pagelet_config(name='audio-conversions.html', context=IAudio, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
@implementer(IMediaConversionsForm)
class AudioConversionsEditForm(AdminDialogEditForm):
    """Audio conversions display form"""

    prefix = 'audio_conversions.'

    legend = _("Selected audio conversions")
    icon_css_class = 'fa fa-fw fa-compress'

    fields = field.Fields(Interface)
    buttons = button.Buttons(IMediaConversionsFormButtons)
    edit_permission = MANAGE_PERMISSION
    ajax_handler = 'restart-audio-conversions.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def check_mode(self):
        if not self.request.has_permission(self.edit_permission, self.getContent()):
            self.mode = DISPLAY_MODE

    def updateActions(self):
        super(AudioConversionsEditForm, self).updateActions()
        if 'convert' in self.actions:
            self.actions['convert'].addClass('btn-primary')

    def applyChanges(self, data):
        media = self.getContent()
        ITransactionManager(media).get().addAfterCommitHook(check_media_conversion, kws={'media': media})
        return True


@view_config(name='restart-audio-conversions.json', context=IAudio, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class AudioConversionsAJAXEditForm(AJAXEditForm, AudioConversionsEditForm):
    """Audio conversions edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Media conversions restarted!"))
        }


@viewlet_config(name='audio-conversions-prefix', context=IAudio, layer=IAdminLayer,
                view=AudioConversionsEditForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/audio-conversions.pt')
class AudioConversionsViewletsPrefix(Viewlet):
    """Audio conversions display form viewlets prefix"""

    @property
    def converters(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for converter_name in utility.audio_formats:
                converter = query_utility(IMediaConverter, name=converter_name)
                if converter is not None:
                    yield converter_name, converter

    def has_conversion(self, converter):
        conversions = IMediaConversions(self.context)
        name = 'audio.{extension}'.format(extension=converter.format)
        return name in conversions
